@@ -9,8 +9,9 @@ import os
 import tempfile
 import zipfile
 from pathlib import Path
-from typing import List, Optional
+from typing import Any, List, Optional
 
+import py7zr
 from natsort import natsorted, ns
 from PIL import Image
 
@@ -39,11 +40,11 @@ class ZipArchiver:
         except zipfile.BadZipfile as bad_zip_error:
             logger.exception(f"bad zipfile [{bad_zip_error}]: {self.path} :: {archive_file}")
             zip_file.close()
-            raise OSError
+            raise OSError from bad_zip_error
         except Exception as exception_error:
             logger.exception(f"bad zipfile [{exception_error}]: {self.path} :: {archive_file}")
             zip_file.close()
-            raise OSError
+            raise OSError from exception_error
         finally:
             zip_file.close()
         return data
@@ -52,7 +53,7 @@ class ZipArchiver:
         """Returns a boolean when attempting to remove a file from an archive"""
 
         try:
-            self.rebuild_zipfile([archive_file])
+            self._rebuild_zipfile([archive_file])
         except zipfile.BadZipfile:
             return False
         else:
@@ -63,7 +64,7 @@ class ZipArchiver:
         #  zip archive w/o the indicated file. Very sucky, but maybe
         # another solution can be found
         try:
-            self.rebuild_zipfile([archive_file])
+            self._rebuild_zipfile([archive_file])
 
             # now just add the archive file as a new one
             zip_file = zipfile.ZipFile(
@@ -88,7 +89,7 @@ class ZipArchiver:
             logger.exception(f"Unable to get zipfile list [{exception_error}]: {self.path}")
             return []
 
-    def rebuild_zipfile(self, exclude_list: List[str]) -> None:
+    def _rebuild_zipfile(self, exclude_list: List[str]) -> None:
         """Zip helper func
 
         This recompresses the zip archive, without the files in the exclude_list
@@ -112,6 +113,125 @@ class ZipArchiver:
         os.rename(tmp_name, self.path)
 
 
+# ------------------------------------------------------------------
+class SevenZipArchiver:
+
+    """7Z implementation"""
+
+    def __init__(self, path: Path) -> None:
+        self.path = path
+
+    def read_archive_file(self, archive_file: str):
+        data = ""
+        try:
+            with py7zr.SevenZipFile(self.path, "r") as zf:
+                if r := zf.read([archive_file]):
+                    data = r[archive_file].read()
+                else:
+                    logger.warning(f"No data in 7zipfile: {self.path} :: {archive_file}")
+                    raise OSError
+        except py7zr.Bad7zFile as e:
+            logger.warning(f"bad 7zip file [{e}]: {self.path} :: {archive_file}")
+            raise OSError from e
+        except Exception as e:
+            logger.warning(f"bad 7zip file [{e}]: {self.path} :: {archive_file}")
+            raise OSError from e
+
+        return data
+
+    def remove_archive_file(self, archive_file: str) -> bool:
+        try:
+            self._rebuild_seven_zipfile([archive_file])
+        except py7zr.Bad7zFile:
+            return False
+        else:
+            return True
+
+    def write_archive_file(self, archive_file: str, data: str) -> bool:
+        try:
+            files = self.get_archive_filename_list()
+            if archive_file in files:
+                self._rebuild_seven_zipfile([archive_file])
+
+            # now just add the archive file as a new one
+            with py7zr.SevenZipFile(self.path, "a") as zf:
+                zf.writestr(data, archive_file)
+            return True
+        except py7zr.Bad7zFile:
+            return False
+
+    def get_archive_filename_list(self) -> List[str]:
+        try:
+            with py7zr.SevenZipFile(self.path, "r") as zf:
+                namelist = zf.getnames()
+
+            return namelist
+        except Exception as e:
+            logger.warning("Unable to get 7zip file list [%s]: %s", e, self.path)
+            return []
+
+    def _rebuild_seven_zipfile(self, exclude_list: List[str]) -> None:
+        """Zip helper func
+
+        This recompresses the zip archive, without the files in the exclude_list
+        """
+        tmp_fd, tmp_name = tempfile.mkstemp(dir=self.path.parent)
+        os.close(tmp_fd)
+
+        try:
+            with py7zr.SevenZipFile(self.path, "r") as zip:
+                targets = [f for f in zip.getnames() if f not in exclude_list]
+            with py7zr.SevenZipFile(self.path, "r") as zin:
+                with py7zr.SevenZipFile(tmp_name, "w") as zout:
+                    if r := zin.read(targets):
+                        for fname, bio in r.items():
+                            zout.writef(bio, fname)
+        except Exception as e:
+            logger.warning("Exception[%s]: %s", e, self.path)
+            return
+
+        # replace with the new file
+        self.path.unlink()
+        os.rename(tmp_name, self.path)
+
+    def copy_from_archive(self, other_archive) -> bool:
+        """Replace the current zip with one copied from another archive"""
+        try:
+            with py7zr.SevenZipFile(self.path, "w") as zout:
+                for fname in other_archive.get_archive_filename_list():
+                    if data := other_archive.read_archive_file(fname):
+                        zout.writestr(data, fname)
+        except Exception as e:
+            logger.warning("Error while copying to %s: %s", self.path, e)
+            return False
+        else:
+            return True
+
+
+# ------------------------------------------------------------------
+class UnknownArchiver:
+
+    """Unknown implementation"""
+
+    def __init__(self, path: Path) -> None:
+        self.path = path
+
+    def read_archive_file(self, archive_file: str) -> Any:
+        return Any
+
+    def write_archive_file(self, archive_file: str, data: str) -> bool:
+        return False
+
+    def remove_archive_file(self, archive_file: str) -> bool:
+        return False
+
+    def get_archive_filename_list(self) -> List[str]:
+        return []
+
+    def copy_from_archive(self) -> bool:
+        return False
+
+
 # ------------------------------------------------------------------
 
 
@@ -120,9 +240,9 @@ class ComicArchive:
     """Comic Archive implementation"""
 
     class ArchiveType:
-        """Types of archives supported. Currently only .cbz"""
+        """Types of archives supported. Currently only .cbz and .cb7"""
 
-        zip, unknown = list(range(2))
+        zip, sevenzip, unknown = list(range(3))
 
     def __init__(self, path: Path) -> None:
         self.path = path
@@ -133,8 +253,15 @@ class ComicArchive:
         self.page_list: Optional[List[str]] = None
         self.metadata: Optional[GenericMetadata] = None
 
-        self.archive_type: int = self.ArchiveType.zip
-        self.archiver = ZipArchiver(self.path)
+        if self.sevenzip_test():
+            self.archive_type: int = self.ArchiveType.sevenzip
+            self.archiver = SevenZipArchiver(self.path)
+        elif self.zip_test():
+            self.archive_type: int = self.ArchiveType.zip
+            self.archiver = ZipArchiver(self.path)
+        else:
+            self.archive_type = self.ArchiveType.unknown
+            self.archiver = UnknownArchiver(self.path)
 
     def reset_cache(self) -> None:
         """Clears the cached data"""
@@ -143,11 +270,19 @@ class ComicArchive:
         self.page_list = None
         self.metadata = None
 
+    def sevenzip_test(self) -> bool:
+        """Tests whether an archive is a sevenzipfile"""
+        return py7zr.is_7zfile(self.path)
+
     def zip_test(self) -> bool:
         """Tests whether an archive is a zipfile"""
 
         return zipfile.is_zipfile(self.path)
 
+    def is_sevenzip(self) -> bool:
+        """Returns a boolean as to whether an archive is a sevenzipfile"""
+        return self.archive_type == self.ArchiveType.sevenzip
+
     def is_zip(self) -> bool:
         """Returns a boolean as to whether an archive is a zipfile"""
 
@@ -164,7 +299,7 @@ class ComicArchive:
     def seems_to_be_a_comic_archive(self) -> bool:
         """Returns a boolean as to whether the file is a comic archive"""
 
-        return bool((self.is_zip()) and (self.get_number_of_pages() > 0))
+        return bool((self.is_zip() or self.is_sevenzip()) and (self.get_number_of_pages() > 0))
 
     def get_page(self, index: int) -> Optional[bytes]:
         """Returns an image(page) from an archive"""
@@ -214,9 +349,9 @@ class ComicArchive:
             # make a sub-list of image files
             self.page_list = []
             for name in files:
-                name_path = Path(name)
-                if self.is_image(name_path):
-                    self.page_list.append(name)
+                name_str = str(name)
+                if self.is_image(Path(name_str)):
+                    self.page_list.append(name_str)
 
         return self.page_list
 
@@ -260,7 +395,7 @@ class ComicArchive:
             raw_metadata = None
         return raw_metadata
 
-    def write_metadata(self, metadata: GenericMetadata) -> bool:
+    def write_metadata(self, metadata: Optional[GenericMetadata]) -> bool:
         """Write the metadata to the archive"""
 
         if metadata is None:
@@ -291,15 +426,14 @@ class ComicArchive:
         """Checks to see if the archive has metadata"""
 
         if self.has_md is None:
+            self.has_md = bool(
+                self.seems_to_be_a_comic_archive()
+                and (
+                    not self.seems_to_be_a_comic_archive()
+                    or self.ci_xml_filename in self.archiver.get_archive_filename_list()
+                )
+            )
 
-            if (
-                not self.seems_to_be_a_comic_archive()
-                or self.seems_to_be_a_comic_archive()
-                and self.ci_xml_filename not in self.archiver.get_archive_filename_list()
-            ):
-                self.has_md = False
-            else:
-                self.has_md = True
         return self.has_md
 
     def apply_archive_info_to_metadata(
@@ -307,7 +441,7 @@ class ComicArchive:
     ) -> None:
         """Apply page information from the archive to the metadata"""
 
-        metadata.page_count = self.get_number_of_pages()
+        metadata.page_count = str(self.get_number_of_pages())
 
         if calc_page_sizes:
             for page in metadata.pages:
@@ -353,3 +487,11 @@ class ComicArchive:
         metadata.is_empty = False
 
         return metadata
+
+    def export_as_cb7(self, new_7zip_filename: Path) -> bool:
+        if self.archive_type == self.ArchiveType.zip:
+            zip_archiver = SevenZipArchiver(new_7zip_filename)
+        else:
+            return False
+
+        return zip_archiver.copy_from_archive(self.archiver)
