@@ -87,7 +87,7 @@ class ComicInfoXml:
         i = "\n" + level * "  "
         if elem:
             if not elem.text or not elem.text.strip():
-                elem.text = i + "  "
+                elem.text = f"{i}  "
             if not elem.tail or not elem.tail.strip():
                 elem.tail = i
             for elem in elem:
@@ -232,10 +232,7 @@ class ComicInfoXml:
         # Helper function
         def xlate(tag: str) -> Optional[str]:
             node = root.find(tag)
-            if node is not None:
-                return node.text
-            else:
-                return None
+            return node.text if node is not None else None
 
         metadata.series = xlate("Series")
         metadata.title = xlate("Title")
