@@ -18,8 +18,8 @@ def get_recursive_filelist(pathlist: List[Path]) -> List[Path]:
     for path_str in pathlist:
         path = Path(path_str)
         if path.is_dir():
-            for filename in path.rglob("*.[cC][bB][zZ]"):
-                filelist.append(filename)
+            for format in ["*.cbz", "*.cb7"]:
+                filelist.extend(iter(path.rglob(format)))
         else:
             filelist.append(path)
 
@@ -43,8 +43,33 @@ def remove_articles(text: str) -> str:
 
     :param str text: A string with articles (ex. 'and', 'a', 'the').
     """
-    articles = ["and", "a", "&", "issue", "the"]
-    new_text = "".join(word + " " for word in text.split(" ") if word.lower() not in articles)
+    articles = [
+        "&",
+        "a",
+        "am",
+        "an",
+        "and",
+        "as",
+        "at",
+        "be",
+        "but",
+        "by",
+        "for",
+        "if",
+        "is",
+        "issue",
+        "it",
+        "it's",
+        "its",
+        "itself",
+        "of",
+        "or",
+        "so",
+        "the",
+        "the",
+        "with",
+    ]
+    new_text = "".join(f"{word} " for word in text.split(" ") if word.lower() not in articles)
 
     new_text = new_text[:-1]
 
