@@ -36,11 +36,17 @@ def test_unique_name(tmp_path):
     assert result == expected_result
 
 
-def test_recursive_list_with_file(tmp_path):
+def test_recursive_list_with_file(tmp_path: Path) -> None:
+    temp_cb7 = tmp_path / "foo.cb7"
+    temp_cb7.write_text("blah")
     temp_file = tmp_path / "test.cbz"
-    expected_result = [temp_file]
-    file_list = [temp_file]
-    result = utils.get_recursive_filelist(file_list)
+    temp_file.write_text("foo")
+    # The following file should be *excluded* from results
+    temp_txt = tmp_path / "fail.txt"
+    temp_txt.write_text("yikes")
+
+    expected_result = [temp_cb7, temp_file]
+    result = utils.get_recursive_filelist([tmp_path])
 
     assert result == expected_result
 
